//
//  App.swift
//  MinGraviditet
//
//  Created by Thomas Kalhøj Clemensen on 08/10/2020.
//

import Foundation
import TIM
import TriforkSwiftDependencyInjection
import TriforkSwiftLogger
import UIKit
import Combine
import ServiceInterfaces
import MockServices

enum BuildConfiguration: String {
    case debug = "Debug"
    case test = "Test"
    case testMock = "TestMock"
    case release = "Release"
    case unknown = "Unknown"
}

struct Configuration {
    let backendUrl: String
    let useMock: Bool
    let loggerConfig: TriforkLoggerConfig
    let tim: TIMConfiguration
}

final class App {
    static let buildConfiguration = BuildConfiguration(rawValue: Bundle.main.buildConfiguration) ?? .unknown
    static let versionString: String = Bundle.main.versionString
    static let configuration = Configuration.build(for: buildConfiguration)

    private static let multiLogger = MultiLogger(loggers: [])

    /// Used to present OpenID Connect login view
    static var windowRootViewController: UIViewController? {
        UIApplication.topViewController()
    }

    static func buildDependencies() {
        setupLoggers()
        setupTriforkIdentityManager()

        Resolver.shared.register(Logger.self) { multiLogger }
        Resolver.shared.register(TIMDataStorage.self) { TIM.storage }
        Resolver.shared.register(TIMAuth.self) { TIM.auth }

        if configuration.useMock {
            addMockServices()
        } else {
            addServices()
        }
    }

    private static func setupLoggers() {
        var loggers: [LoggerProtocol] = [TriforkLogger(config: configuration.loggerConfig)]
        if buildConfiguration == .debug {
            loggers.append(ErrorFlashLogger())
        }
        multiLogger.loggers = loggers
    }

    private static func addServices() {
        // Use TriforkSwiftDependencyInjection
    }

    private static func addMockServices() {
        // Use TriforkSwiftDependencyInjection
        Resolver.shared.register(TokenService.self) { () in
            MockTokenService()
        }
    }

    private static func setupTriforkIdentityManager() {
        TIM.configure(configuration: configuration.tim)
    }
}

private extension Configuration {
    //swiftlint:disable:next function_body_length
    static func build(for buildConfiguration: BuildConfiguration) -> Configuration {
        let timConfiguration = TIMConfiguration.forBuildConfiguration(buildConfiguration)

        switch buildConfiguration {
        case .debug:
            return Configuration(
                backendUrl: "",
                useMock: true,
                loggerConfig: TriforkLoggerConfig(
                    defaultCategory: buildConfiguration.rawValue,
                    minimumLogLevel: .default,
                    isDevelopmentInfoEnabled: false,
                    isEmojisEnabled: true
                ),
                tim: timConfiguration
            )
        case .test:
            return Configuration(
                backendUrl: "",
                useMock: false,
                loggerConfig: TriforkLoggerConfig(
                    defaultCategory: buildConfiguration.rawValue,
                    minimumLogLevel: .info,
                    isDevelopmentInfoEnabled: true,
                    isEmojisEnabled: true
                ),
                tim: timConfiguration
            )
        case .testMock:
            return Configuration(
                backendUrl: "",
                useMock: true,
                loggerConfig: TriforkLoggerConfig(
                    defaultCategory: buildConfiguration.rawValue,
                    minimumLogLevel: .info,
                    isDevelopmentInfoEnabled: true,
                    isEmojisEnabled: true
                ),
                tim: timConfiguration
            )
        case .release:
            return Configuration(
                backendUrl: "",
                useMock: false,
                loggerConfig: TriforkLoggerConfig(
                    defaultCategory: buildConfiguration.rawValue,
                    minimumLogLevel: .error,
                    isDevelopmentInfoEnabled: true,
                    isEmojisEnabled: true
                ),
                tim: timConfiguration
            )
        case .unknown:
            fatalError("App was started with unknown build configuration 🤯")
        }
    }
}
