//
//  SceneDelegate.swift
//  MinGraviditet
//
//  Created by Thomas Kalhøj Clemensen on 08/10/2020.
//

import UIKit
import SwiftUI
import TIM
import ServiceInterfaces
import TriforkSwiftDependencyInjection

extension Notification.Name {
    final class SceneDelegate {
        static let changeRootViewController = Notification.Name("SceneDelegate.ChangeRootViewController")
    }
}

final class SceneDelegate: UIResponder, UIWindowSceneDelegate {

    var window: UIWindow?

    private var cancelBag = CancelBag()

    @Inject private var logger: Logger
    @Inject private var timDataStorage: TIMDataStorage
    @Inject private var timAuth: TIMAuth

    func scene(_ scene: UIScene, willConnectTo session: UISceneSession, options connectionOptions: UIScene.ConnectionOptions) {
        guard let windowScene = scene as? UIWindowScene else { return }

        let window = UIWindow(windowScene: windowScene)
        self.window = window

        listenForNotfications()

        if timDataStorage.availableUserIds.isEmpty {
            let onboarding = UIHostingController(rootView: OnboardingView())
            changeRootViewController(onboarding, animated: false)
        } else {
            changeRootViewController(SessionLoadingViewController(), animated: false)

            timAuth.accessToken()
                .sink { (completion) in
                    switch completion {
                    case .failure(let error):
                        self.logger.info("Failed to receive access token, the session must be invalid: \(error.localizedDescription)")
                        if let userId = self.timDataStorage.availableUserIds.first {
                            let login = UIHostingController(rootView: LoginView(userId: userId))
                            self.changeRootViewController(login, animated: false)
                        } else {
                            self.logger.fault("This state should never happen, but we are making a fallback to onboarding if it does happen!")
                            let onboarding = UIHostingController(rootView: OnboardingView())
                            self.changeRootViewController(onboarding, animated: false)
                        }
                    case .finished:
                        self.changeRootViewController(TabBarController(), animated: true)
                    }
                } receiveValue: { at in
                    self.logger.debug("We will have a valid AT:\n\(at)")
                }
                .store(in: &cancelBag)
        }

        window.makeKeyAndVisible()
    }

    func sceneDidDisconnect(_ scene: UIScene) {
        // Called as the scene is being released by the system.
        // This occurs shortly after the scene enters the background, or when its session is discarded.
        // Release any resources associated with this scene that can be re-created the next time the scene connects.
        // The scene may re-connect later, as its session was not necessarily discarded (see `application:didDiscardSceneSessions` instead).
    }

    func sceneDidBecomeActive(_ scene: UIScene) {
        // Called when the scene has moved from an inactive state to an active state.
        // Use this method to restart any tasks that were paused (or not yet started) when the scene was inactive.
    }

    func sceneWillResignActive(_ scene: UIScene) {
        // Called when the scene will move from an active state to an inactive state.
        // This may occur due to temporary interruptions (ex. an incoming phone call).
    }

    func sceneWillEnterForeground(_ scene: UIScene) {
        // Called as the scene transitions from the background to the foreground.
        // Use this method to undo the changes made on entering the background.
    }

    func sceneDidEnterBackground(_ scene: UIScene) {
        // Called as the scene transitions from the foreground to the background.
        // Use this method to save data, release shared resources, and store enough scene-specific state information
        // to restore the scene back to its current state.
    }

    func scene(_ scene: UIScene, openURLContexts URLContexts: Set<UIOpenURLContext>) {
        for url: URL in URLContexts.map({ $0.url }) {
            TIM.auth.handleRedirect(url: url)
        }
    }

    // MARK: -
    private func listenForNotfications() {
        NotificationCenter.default
            .publisher(for: Notification.Name.SceneDelegate.changeRootViewController)
            .compactMap({ $0.object as? NotificationMessage })
            .sink(receiveValue: { (message: NotificationMessage) in
                self.handleNotification(message)
            })
            .store(in: &cancelBag)
    }

    private func changeRootViewController(_ rootViewController: UIViewController, animated: Bool) {
        guard let window = window else {
            fatalError("Failed to change rootViewController, since the windows was nil")
        }
        window.rootViewController = rootViewController
        window.makeKeyAndVisible()

        if animated {
            UIView.transition(
                with: window,
                duration: 0.2,
                options: .transitionCrossDissolve,
                animations: {},
                completion: nil
            )

        }
    }
}

extension SceneDelegate {
    enum NotificationMessage {
        case beginRegisterFlow(userId: String)
        case loginSucceded(userId: String)
    }

    private func handleNotification(_ notification: NotificationMessage) {
        switch notification {
        case .beginRegisterFlow(let userId):
            let choosePin = UIHostingController(rootView: ChoosePinCodeView(userId: userId))
            changeRootViewController(choosePin, animated: true)
        case .loginSucceded:
            changeRootViewController(TabBarController(), animated: true)
        }
    }
}
