import SwiftUI
import UIKit
import WebKit
import TriforkSwiftDependencyInjection

class UIScrollViewViewController: UIViewController {

    let scrollView = UIScrollView()

    var hostingController: UIHostingController<AnyView> = UIHostingController(rootView: AnyView(EmptyView()))

    override func viewDidLoad() {
        super.viewDidLoad()
        self.view.addSubview(self.scrollView)
        self.pinEdges(of: self.scrollView, to: self.view)

        self.hostingController.willMove(toParent: self)
        self.scrollView.addSubview(self.hostingController.view)
        self.pinEdges(of: self.hostingController.view, to: self.scrollView)
        self.hostingController.didMove(toParent: self)

    }

    func pinEdges(of viewA: UIView, to viewB: UIView) {
        viewA.translatesAutoresizingMaskIntoConstraints = false
        viewB.addConstraints([
            viewA.leadingAnchor.constraint(equalTo: viewB.leadingAnchor),
            viewA.trailingAnchor.constraint(equalTo: viewB.trailingAnchor),
            viewA.topAnchor.constraint(equalTo: viewB.topAnchor),
            viewA.bottomAnchor.constraint(equalTo: viewB.bottomAnchor)
        ])
    }

}

struct SUIScrollView<Content: View>: UIViewControllerRepresentable {

    var content: () -> Content
    @Binding var viewModel: SUIScrollViewViewModel

    init(viewModel: Binding<SUIScrollViewViewModel>, @ViewBuilder content: @escaping () -> Content) {
        self.content = content
        self._viewModel = viewModel
    }

    func makeUIViewController(context: Context) -> UIScrollViewViewController {
        let vc = UIScrollViewViewController()
        vc.hostingController.rootView = AnyView(self.content())
        return vc
    }

    func updateUIViewController(_ viewController: UIScrollViewViewController, context: Context) {
        viewController.hostingController.rootView = AnyView(self.content())
        viewController.scrollView.delegate = context.coordinator
        viewController.scrollView.isPagingEnabled = viewModel.isPaged
        viewController.scrollView.showsVerticalScrollIndicator = viewModel.showIndicators
        viewController.scrollView.showsHorizontalScrollIndicator = viewModel.showIndicators
        viewController.scrollView.bounces = viewModel.bounces
    }

    func makeCoordinator() -> Coordinator {
        Coordinator(currentPage: $viewModel.currentPage)
    }

    /// Coordinator to handle NSObject requirement for delegate.
    final class Coordinator : NSObject, UIScrollViewDelegate {

        @Binding var currentPage: Int

        init(currentPage: Binding<Int>?) {
            self._currentPage = currentPage ?? .constant(-1)
        }

        func scrollViewDidScroll(_ scrollView: UIScrollView) {
            if currentPage != scrollView.currentPage {
                currentPage = scrollView.currentPage
            }
        }
    }
}

final class SUIScrollViewViewModel: ObservableObject {

    @Published var currentPage: Int = 0

    var isPaged: Bool
    var bounces: Bool
    var showIndicators: Bool

    init(isPaged: Bool, bounces: Bool, showIndicators: Bool) {
        self.isPaged = isPaged
        self.bounces = bounces
        self.showIndicators = showIndicators
    }
}
