import SwiftUI
import UIKit
import WebKit
import TriforkSwiftDependencyInjection

struct SUIWKWebView: UIViewRepresentable {
    var viewModel: ViewModel

    func makeUIView(context: Context) -> WKWebView {
        WKWebView()
    }

    func updateUIView(_ uiView: WKWebView, context: Context) {
        viewModel.failedToLoad = false
        viewModel.isLoading = true

        uiView.navigationDelegate = context.coordinator
        if uiView.url != viewModel.url { //Don't reload the same url multiple times
            let request = URLRequest(url: viewModel.url)
            uiView.load(request)
        }
    }

    func makeCoordinator() -> Coordinator {
        Coordinator(viewModel: viewModel)
    }

    /// Coordinator to handle NSObject requirement for navigation delegate.
    final class Coordinator : NSObject, WKNavigationDelegate {

        @Inject var logger: Logger
        var viewModel: ViewModel

        init(viewModel: ViewModel) {
            self.viewModel = viewModel
        }

        func webView(_ webView: WKWebView, didCommit navigation: WKNavigation!) {
            logger.default("Starting load of \(webView.url?.absoluteString ?? "<none>")")
        }

        func webView(_ webView: WKWebView, didFinish navigation: WKNavigation!) {
            viewModel.isLoading = false
            logger.default("Finished loading \(webView.url?.absoluteString ?? "<none>")")
        }

        func webView(_ webView: WKWebView, didFail navigation: WKNavigation!, withError error: Error) {
            viewModel.failedToLoad = true
            viewModel.isLoading = false
            logger.default("Failed to load \(webView.url?.absoluteString ?? "<none>")")
        }

        func webView(_ webView: WKWebView, decidePolicyFor navigationAction: WKNavigationAction, decisionHandler: @escaping (WKNavigationActionPolicy) -> Void) {
            //Allow everyting for now
            decisionHandler(.allow)
        }
    }

    final class ViewModel: ObservableObject {
        @Published var isLoading: Bool = false
        @Published var failedToLoad: Bool = false
        let url: URL

        init(url: URL) {
            self.url = url
        }
    }
}
